﻿-- License: LGPL v2.1 (this file specifically)

local _G = _G

local rule = ArkInventoryRules:NewModule( "ArkInventoryRules_Wardrobe" )

local model = CreateFrame("DressUpModel")
model:SetSize(1,1)


local inventoryTypes = {
		["INVTYPE_HEAD"] = 1,
		["INVTYPE_SHOULDER"] = 2,
		["INVTYPE_CLOAK"] = 3,
		["INVTYPE_CHEST"] = 4,
		["INVTYPE_ROBE"] = 4,
		["INVTYPE_BODY"] = 5,
		["INVTYPE_TABARD"] = 6,
		["INVTYPE_WRIST"] = 7,
		["INVTYPE_HAND"] = 8,
		["INVTYPE_WAIST"] = 9,
		["INVTYPE_LEGS"] = 10,
		["INVTYPE_FEET"] = 11,
		["Wands"] = 12,
		["One-Handed Axes"] = 13,
		["One-Handed Swords"] = 14,
		["One-Handed Maces"] = 15,
		["Daggers"] = 16,
		["Fist Weapons"] = 17,
		["INVTYPE_SHIELD"] = 18,
		["INVTYPE_HOLDABLE"] = 19,
		["Two-Handed Axes"] = 20,
		["Two-Handed Swords"] = 21,
		["Two-Handed Maces"] = 22,
		["Staves"] = 23,
		["Polearms"] = 24,
		["Bows"] = 25,
		["Guns"] = 26,
		["Crossbows"] = 27,
		["Warglaives"] = 28,
	}
	do
		local WeaponToCategory = {
			[87066] = 12,
			[19362] = 13,
			[86987] = 14,
			[78476] = 15,
			[86217] = 16,
			[50035] = 17,
			[28298] = 20,
			[59039] = 21,
			[70723] = 22,
			[71559] = 23,
			[24044] = 24,
			[50638] = 25,
			[50444] = 26,
			[47907] = 27,
			[127829] = 28,
		}
		for itemID,_ in pairs(WeaponToCategory) do
			GetItemInfo(itemID)
		end
		
		C_Timer.NewTimer(1,function()
			for itemID,categoryID in pairs(WeaponToCategory) do
				local _,_,_,_,_,_,subclass = GetItemInfo(itemID)
				if subclass then 
					inventoryTypes[ subclass ] = categoryID 
				end
			end
		end)
	end


function rule:OnEnable( )
	local registered
	
	registered = ArkInventoryRules.Register( self, "wardrobeunknown", rule.execute_wardrobeUnknown )
	registered = ArkInventoryRules.Register( self, "wardrobeknown", rule.execute_wardrobeKnown )
	registered = ArkInventoryRules.Register( self, "wardrobeinvalid", rule.execute_wardrobeInvalid )
	
end

function rule.execute_wardrobeUnknown( ... )
	
	-- always check for the hyperlink and that it's an actual item, not a spell (pet/mount)
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end
		
	local fn = "wardrobeUnknown" -- your rule function name, needs to be set so that error messages are readable

	if ArkInventoryRules.Object.sb then
		return false
	end	
	
	local _,_,quality,_,_,class,subclass,_,equipSlot = GetItemInfo(ArkInventoryRules.Object.h)
		
	local slot = (equipSlot == "INVTYPE_TABARD") or (subclass and inventoryTypes[subclass]) or (equipSlot and inventoryTypes[equipSlot])
	
	if not slot then
		return false
	end
	
	local function GetAppearanceDataByItem(item)
		model:SetUnit("player")
		model:Undress()
		model:TryOn(item)
		
		for i=1,20 do
			local appearanceSourceID = model:GetSlotTransmogSources(i)
			if (appearanceSourceID and appearanceSourceID > 0) then
				return C_TransmogCollection.GetAppearanceInfoBySource(appearanceSourceID)
			end
		end 
	end
	 
	function PlayerHasTransmog(item)
				
		local data = GetAppearanceDataByItem(item)
		local isCollected = data and data.appearanceIsCollected or false
				
		return isCollected
	end
		
	local isLearnedItem = PlayerHasTransmog(ArkInventoryRules.Object.h)
	
	if isLearnedItem or ArkInventoryRules.Object.q < 2 then	
		return false
	end
	
	ArkInventory.TooltipSetHyperlink( ArkInventoryRules.Tooltip, ArkInventoryRules.Object.h )	
	
	if ArkInventory.TooltipContains( ArkInventoryRules.Tooltip, TRANSMOGRIFY_TOOLTIP_APPEARANCE_UNKNOWN ) then
		return true
	end
	
	return false	
end

function rule.execute_wardrobeKnown( ... )
	
	-- always check for the hyperlink and that it's an actual item, not a spell (pet/mount)
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end
		
	local fn = "wardrobeKnown" -- your rule function name, needs to be set so that error messages are readable

	if ArkInventoryRules.Object.sb then
		return false
	end	
	
	local _,_,quality,_,_,class,subclass,_,equipSlot = GetItemInfo(ArkInventoryRules.Object.h)
		
	local slot = (equipSlot == "INVTYPE_TABARD") or (subclass and inventoryTypes[subclass]) or (equipSlot and inventoryTypes[equipSlot])
	
	if not slot then
		return false
	end
	
	local function GetAppearanceDataByItem(item)
		model:SetUnit("player")
		model:Undress()
		model:TryOn(item)
		
		for i=1,20 do
			local appearanceSourceID = model:GetSlotTransmogSources(i)
			if (appearanceSourceID and appearanceSourceID > 0) then
				return C_TransmogCollection.GetAppearanceInfoBySource(appearanceSourceID)
			end
		end 
	end
	 
	function PlayerHasTransmog(item)
				
		local data = GetAppearanceDataByItem(item)
		local isCollected = data and data.appearanceIsCollected or false
				
		return isCollected
	end
		
	local isLearnedItem = PlayerHasTransmog(ArkInventoryRules.Object.h)
		
	if isLearnedItem then
		return true
	end	
	
	return false	
end

function rule.execute_wardrobeInvalid( ... )
	
	-- always check for the hyperlink and that it's an actual item, not a spell (pet/mount)
	if not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end
		
	local fn = "wardrobeInvalid" -- your rule function name, needs to be set so that error messages are readable

	if ArkInventoryRules.Object.sb then
		return false
	end	
	
	local _,_,quality,_,_,class,subclass,_,equipSlot = GetItemInfo(ArkInventoryRules.Object.h)
		
	local slot = (equipSlot == "INVTYPE_TABARD") or (subclass and inventoryTypes[subclass]) or (equipSlot and inventoryTypes[equipSlot])
	
	if not slot then
		return false
	end
	
	local function GetAppearanceDataByItem(item)
		model:SetUnit("player")
		model:Undress()
		model:TryOn(item)
		
		for i=1,20 do
			local appearanceSourceID = model:GetSlotTransmogSources(i)
			if (appearanceSourceID and appearanceSourceID > 0) then
				return C_TransmogCollection.GetAppearanceInfoBySource(appearanceSourceID)
			end
		end 
	end
	 
	function PlayerHasTransmog(item)
				
		local data = GetAppearanceDataByItem(item)
		local isCollected = data and data.appearanceIsCollected or false
				
		return isCollected
	end
		
	local isLearnedItem = PlayerHasTransmog(ArkInventoryRules.Object.h)
	
	if isLearnedItem or ArkInventoryRules.Object.q < 2 then	
		return false
	end
	
	ArkInventory.TooltipSetHyperlink( ArkInventoryRules.Tooltip, ArkInventoryRules.Object.h )	
	
	if not ArkInventory.TooltipContains( ArkInventoryRules.Tooltip, TRANSMOGRIFY_TOOLTIP_APPEARANCE_UNKNOWN ) then
		return true
	end
	
	return false	
end


